package de.own.skill;

import com.github.appreciated.apexcharts.ApexCharts;
import com.github.appreciated.apexcharts.ApexChartsBuilder;
import com.github.appreciated.apexcharts.config.builder.*;
import com.github.appreciated.apexcharts.config.chart.Type;
import com.github.appreciated.apexcharts.config.subtitle.Style;
import com.github.appreciated.apexcharts.config.theme.Mode;
import com.github.appreciated.apexcharts.helper.Series;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.Unit;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.Route;

import java.util.List;

@Route("")
public class PresentationView extends VerticalLayout {
    private final HorizontalLayout backendLayout = new HorizontalLayout();
    private final HorizontalLayout frontendLayout = new HorizontalLayout();
    private final Grid<Row> backendGrid = new Grid<>(Row.class);
    private final Grid<Row> frontendGrid = new Grid<>(Row.class);
    public PresentationView() {
        setSizeFull();
        MyData backendData = initBackendData();
        MyData frontendData = initFrontendData();


        backendGrid.setItems(backendData.getData());
        backendGrid.setMinWidth(500, Unit.PIXELS);
        backendGrid.setMaxWidth(500, Unit.PIXELS);
        backendGrid.setHeightFull();

        frontendGrid.setItems(frontendData.getData());
        frontendGrid.setMinWidth(500, Unit.PIXELS);
        frontendGrid.setMaxWidth(500, Unit.PIXELS);
        frontendGrid.setHeightFull();

        backendLayout.setWidth(500, Unit.PIXELS);
        backendLayout.setHeight(500, Unit.PIXELS);
        backendLayout.add(backendGrid);
        ApexCharts backendChart = initChartByType(backendData, Type.RADAR, "Backend skills");
        backendLayout.add(backendChart);


        frontendLayout.setHeightFull();
        frontendLayout.setMaxWidth(500, Unit.PIXELS);
        frontendLayout.add(frontendGrid);
        ApexCharts frontendChart = initChartByType(frontendData, Type.BAR, "Frontend skills");
        frontendLayout.add(frontendChart);

        UI.getCurrent().getPage().retrieveExtendedClientDetails(details -> {
            if (details.getBodyClientWidth() < 500) {
                add(new VerticalLayout(backendChart, backendGrid, frontendChart, frontendGrid));
                backendGrid.setHeight(500, Unit.PIXELS);
                frontendGrid.setHeight(500, Unit.PIXELS);
            } else {
                add(backendLayout,frontendLayout);
            }
        });

        UI.getCurrent().getPage().addStyleSheet("data:text/css," +
                ".apexcharts-menu-icon { display: none !important; }");
    }

    private ApexCharts initChartByType(MyData data, Type type, String title) {
        String[] names = data.getData().stream().map(Row::getName).toArray(String[]::new);
        Double[] scores = data.getData().stream().map(Row::getScore).toArray(Double[]::new);
        Style style =new Style();
        style.setColor("red");
        ApexCharts chart = new ApexChartsBuilder()
                .withTheme(ThemeBuilder.get().withMode(Mode.LIGHT).build())
                .withChart(ChartBuilder.get().withType(type).build())
                .withTitle(TitleSubtitleBuilder.get().withText(title).withStyle(style).build())
                .withYaxis(YAxisBuilder.get()
                        .withMin(0d)
                        .withMax(9d)
                        .build())
                .withTooltip(TooltipBuilder.get().withTheme("dark").withFillSeriesColor(true).build())
//                .withColors("#142a4d", "#142a4d", "#142a4d")
//                .withColors("red")

                .withLabels(names)
                .withSeries(new Series<>("Scores", scores))
                .build();
        chart.setWidth(500, Unit.PIXELS);
        chart.setHeight(500, Unit.PIXELS);
        return chart;
    }

    private MyData initBackendData() {
        MyData myData = new MyData();
        myData.setData(List.of(
                Row.builder().name("Apache Poi").score(7d).build(),
                Row.builder().name("Flowable").score(5d).build(),
                Row.builder().name("Flyway").score(6d).build(),
                Row.builder().name("Git").score(8d).build(),
                Row.builder().name("Hibernate").score(8d).build(),
                Row.builder().name("Jasper Report").score(6d).build(),
                Row.builder().name("Jenkins").score(3d).build(),
                Row.builder().name("Junit").score(5d).build(),
                Row.builder().name("Liquibase").score(7d).build(),
                Row.builder().name("Mapstruct").score(7d).build(),
                Row.builder().name("Open CSV").score(5d).build(),
                Row.builder().name("QueryDSL").score(6d).build(),
                Row.builder().name("SonarQ").score(3d).build(),
                Row.builder().name("Spring").score(8d).build(),
                Row.builder().name("Spring Data JPA").score(8d).build()
                )
        );
        return myData;
    }

    private MyData initFrontendData() {
        MyData myData = new MyData();
        myData.setData(List.of(
                        Row.builder().name("Primeface").score(7d).build(),
                        Row.builder().name("Swing").score(6d).build(),
                        Row.builder().name("Vaadin").score(7d).build(),
                        Row.builder().name("Vuejs").score(6d).build()
                )
        );
        return myData;
    }
}
